@@ -34,7 +34,8 @@ class BrokerView(BaseHandler):
         try:
             queue_names = ControlHandler.get_active_queue_names()
             if not queue_names:
-                queue_names = set([self.capp.conf.CELERY_DEFAULT_QUEUE])
+                queue_names = set([self.capp.conf.CELERY_DEFAULT_QUEUE]) |\
+                        set([q.name for q in self.capp.conf.CELERY_QUEUES or [] if q.name])
 
             queues = yield broker.queues(sorted(queue_names))
         except Exception as e:
